%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "pluged.h"

/*
 * Structures
 */

enum {
	USBD_NONE,
	USBD_PRODUCT,
	USBD_VENDOR_NO,
	USBD_PRODUCT_NO,
	USBD_RELEASE_NO,
	USBD_DESCR,
	USBD_DEV,
	USBD_ATTACH,
	USBD_DETACH
} current_item = USBD_NONE;

/*
 * Globals
 */

int not_initialised     = 1;
unsigned int line_count = 1;
struct usb_config_head element_list;

/*
 * Functions
 */

struct usb_config_entry * new_empty_usb_entry(void) {
	struct usb_config_entry * ent = malloc(sizeof(struct usb_config_entry));
	ent->used_fields = 0;
	*(ent->descr)    = 0;
	*(ent->dev)      = 0;
	*(ent->attach)   = 0;
	*(ent->detach)   = 0;
	*(ent->product)  = 0;
	ent->product_no  = 0;
	ent->vendor_no   = 0;
	ent->release_no  = 0;
	return ent;
}

void error(void) {
	fprintf(stderr,"Error line %d.\n",line_count);
	exit(EXIT_FAILURE);
}
/*
#define error() { fprintf(stderr,"(%s, l.%d) Error line %d.\n",__FILE__,__LINE__,line_count); exit(EXIT_FAILURE); }
*/
%}

HEXVAL 0x[A-Za-z0-9]+
STRING ["].*["]
COMMENT ^#.*$

%%
%array

{STRING} {
	struct usb_config_entry *e;
	char *param = yytext + 1;
	yytext[strlen(yytext)-1] = 0;

	switch(current_item) {
		case USBD_DESCR:
			if(not_initialised) {
				not_initialised = 0;
			} else {
				e = new_empty_usb_entry();
				SLIST_INSERT_HEAD(&element_list,e,next);
			}
			e = SLIST_FIRST( &element_list );
			strncpy(e->descr,param,USBD_MAX_DESCRLEN);
			break;
		case USBD_DEV:
			if(not_initialised) error();
			e = SLIST_FIRST( &element_list );
			strncpy(e->dev,param,USB_MAX_DEVNAMELEN);
			regcomp(&(e->dev_regex),e->dev,REG_NOSUB|REG_EXTENDED);
			e->used_fields |= USEFIELD_DEV;
			break;
		case USBD_PRODUCT:
			if(not_initialised) error();
			e = SLIST_FIRST( &element_list );
			strncpy(e->product,param,USB_MAX_STRING_LEN);
			regcomp(&(e->product_regex),e->product,REG_NOSUB|REG_EXTENDED);
			e->used_fields |= USEFIELD_PRODUCT;
			break;
		case USBD_ATTACH:
			if(not_initialised) error();
			e = SLIST_FIRST( &element_list );
			strncpy(e->attach,param,USBD_MAX_CMDLEN);
			break;
		case USBD_DETACH:
			if(not_initialised) error();
			e = SLIST_FIRST( &element_list );
			strncpy(e->detach,param,USBD_MAX_CMDLEN);
			break;
		case USBD_NONE:
		default:
			error();
	}
}

{HEXVAL} {
	struct usb_config_entry *e;
	switch(current_item) {
		case USBD_VENDOR_NO:
			if(not_initialised) error();
			e = SLIST_FIRST( &element_list );
			sscanf(yytext,"%X",(unsigned int*)&(e->vendor_no));
			e->used_fields |= USEFIELD_VENDOR_NO;
			break;
		case USBD_RELEASE_NO:
			if(not_initialised) error();
			e = SLIST_FIRST( &element_list );
			sscanf(yytext,"%X",(unsigned int*)&(e->release_no));
			e->used_fields |= USEFIELD_RELEASE_NO;
			break;
		case USBD_PRODUCT_NO:
			if(not_initialised) error();
			e = SLIST_FIRST( &element_list );
			sscanf(yytext,"%X",(unsigned int*)&(e->product_no));
			e->used_fields |= USEFIELD_PRODUCT_NO;
			break;
		default:
			error();
	}
}

description {
	current_item = USBD_DESCR;
}
devname {
	current_item = USBD_DEV;
}
attach {
	current_item = USBD_ATTACH;
}
detach {
	current_item = USBD_DETACH;
}
vendorID {
	current_item = USBD_VENDOR_NO;
}
productID {
	current_item = USBD_PRODUCT_NO;
}
product {
	current_item = USBD_PRODUCT;
}
release {
	current_item = USBD_RELEASE_NO;
}

\n { current_item = USBD_NONE; ++line_count; }
{COMMENT}
" "|"	" 
. {
	printf("'%c'=%d(%d)\n",*yytext,*yytext,line_count);
	error();
}


%%
struct usb_config_head read_config(char *filename) {
	struct usb_config_entry *e = new_empty_usb_entry();
	SLIST_INIT(&element_list);
	SLIST_INSERT_HEAD(&element_list,e,next);

	yyin = fopen(filename,"r");
	if( yyin == NULL ) {
		perror(filename);
		exit(EXIT_FAILURE);
	}
	yylex();
	fclose(yyin);
	if(not_initialised)
		error();
	return element_list;
}
/*
int main(int argc, char **argv)
{
	struct usb_config_head lel;
	struct usb_config_entry *e;

	lel = read_config(argv[1]);
	SLIST_FOREACH( e, &lel, next) {
		printf("--------\ndescr(%s)\ndev(%s)\nattach(%s)\ndetach(%s)\n-------\n",e->descr,e->dev,e->attach,e->detach);
	}
	return 0;
}
*/
